pub struct TombstoneManager {
kv_backend: KvBackendRef,
tombstone_prefix: String,
}Expand description
TombstoneManager provides the ability to:
- logically delete values
- restore the deleted values
The tombstone mechanism is primarily used for table metadata deletion.
When a table is logically deleted, its associated metadata keys are moved
to a tombstone area by prepending a prefix (default is __tombstone/).
All involved keys in the tombstone mechanism:
__table_name/{catalog}/{schema}/{table_name}: Maps table name to table ID.__table_info/{table_id}: Stores table information/metadata.__table_route/{table_id}: Stores table routing information.__table_repart/{table_id}: Stores table repartition information.__dn_table/{datanode_id}/{table_id}: Maps datanodes to table regions.__topic_region/{topic_name}/{region_id}: Maps regions to WAL topics.
These keys are moved to:
__tombstone/__table_name/...__tombstone/__table_info/...__tombstone/__table_route/...__tombstone/__table_repart/...__tombstone/__dn_table/...__tombstone/__topic_region/...
Fields
kv_backend: KvBackendReftombstone_prefix: StringImplementations
Sourceimpl TombstoneManager
impl TombstoneManager
Sourcepub fn new(kv_backend: KvBackendRef) -> Self
pub fn new(kv_backend: KvBackendRef) -> Self
Returns TombstoneManager.
Sourcepub fn new_with_prefix(kv_backend: KvBackendRef, prefix: &str) -> Self
pub fn new_with_prefix(kv_backend: KvBackendRef, prefix: &str) -> Self
Returns TombstoneManager with a custom tombstone prefix.
pub fn to_tombstone(&self, key: &[u8]) -> Vec<u8> ⓘ
Sourcepub fn strip_tombstone_prefix<'a>(
&self,
tombstone_key: &'a [u8],
) -> Result<&'a [u8]>
pub fn strip_tombstone_prefix<'a>( &self, tombstone_key: &'a [u8], ) -> Result<&'a [u8]>
Removes the tombstone prefix from a tombstoned key.
Sourcepub async fn get(&self, key: &[u8]) -> Result<Option<KeyValue>>
pub async fn get(&self, key: &[u8]) -> Result<Option<KeyValue>>
Gets a single tombstoned value by its original key.
Sourcepub async fn batch_get(
&self,
keys: &[Vec<u8>],
) -> Result<HashMap<Vec<u8>, KeyValue>>
pub async fn batch_get( &self, keys: &[Vec<u8>], ) -> Result<HashMap<Vec<u8>, KeyValue>>
Gets tombstoned values by their original keys.
Sourcepub fn tombstones(&self) -> BoxStream<'static, Result<KeyValue>>
pub fn tombstones(&self) -> BoxStream<'static, Result<KeyValue>>
Streams all tombstoned key-value pairs.
Sourcepub fn tombstoned_table_names(&self) -> BoxStream<'static, Result<KeyValue>>
pub fn tombstoned_table_names(&self) -> BoxStream<'static, Result<KeyValue>>
Streams tombstoned table-name entries only.
Sourcefn scan_prefix(&self, prefix: Vec<u8>) -> BoxStream<'static, Result<KeyValue>>
fn scan_prefix(&self, prefix: Vec<u8>) -> BoxStream<'static, Result<KeyValue>>
Streams tombstoned entries under the provided prefix.
Sourcefn build_move_value_txn(
&self,
src_key: Vec<u8>,
value: Vec<u8>,
dest_key: Vec<u8>,
) -> (Txn, impl FnMut(&mut TxnOpGetResponseSet) -> Option<Vec<u8>>)
fn build_move_value_txn( &self, src_key: Vec<u8>, value: Vec<u8>, dest_key: Vec<u8>, ) -> (Txn, impl FnMut(&mut TxnOpGetResponseSet) -> Option<Vec<u8>>)
Moves value to dest_key.
Puts value to dest_key if the value of src_key equals value.
Otherwise retrieves the value of src_key.
async fn move_values_inner( &self, keys: &[Vec<u8>], dest_keys: &[Vec<u8>], ) -> Result<usize>
fn max_txn_ops(&self) -> usize
Sourceasync fn move_values(
&self,
keys: Vec<Vec<u8>>,
dest_keys: Vec<Vec<u8>>,
) -> Result<usize>
async fn move_values( &self, keys: Vec<Vec<u8>>, dest_keys: Vec<Vec<u8>>, ) -> Result<usize>
Moves values to dest_key.
Returns the number of keys that were moved.
Sourcepub async fn create(&self, keys: Vec<Vec<u8>>) -> Result<usize>
pub async fn create(&self, keys: Vec<Vec<u8>>) -> Result<usize>
Creates tombstones for keys.
Preforms to:
- deletes origin values.
- stores tombstone values.
Returns the number of keys that were moved.